//------------------------------------------------------------------
// CONTEC PCI RESOURCE RECOGNIZING TOOL FOR MS-DOS
// GETCONF.C
//												CONTEC Co.,Ltd
//												2002.04.18
//------------------------------------------------------------------
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include "getpci.h"

//------------------------------------------------------------------
typedef struct _PCI_SLOT_NUMBER {
	union {
		struct {
			unsigned char FunctionNumber	:	3;
			unsigned char DeviceNumber		:	5;
		} bits;
		unsigned char AsChar;
	} num;
} SLOT_NUMBER;

//-------------------------------------------------------------------
// Sub routine : Class name translate (PCI Local Bus Specification Revision 2.3)
// char *GetPciClassName(unsigned long Reg)
// Parameters:
//			Reg		:	configuration register value register.:+8 to +11
// return:
//			constant string pointer of class name string.
//-------------------------------------------------------------------
static char *GetPciClassName(unsigned long Reg)
{
	static char szBuf[100];
	switch(Reg & 0xffffff00) {
	case 0x00000000:	return "All currently implemented devices except VGA-compatible devices";
	case 0x00010000:	return "VGA-compatible device";
	case 0x01000000:	return "SCSI bus controller";
	case 0x01010000:	return "IDE controller";
	case 0x01010100:	return "IDE controller";
	case 0x01010200:	return "IDE controller";
	case 0x01010300:	return "IDE controller";
	case 0x01010400:	return "IDE controller";
	case 0x01010500:	return "IDE controller";
	case 0x01010600:	return "IDE controller";
	case 0x01010700:	return "IDE controller";
	case 0x01010800:	return "IDE controller";
	case 0x01010900:	return "IDE controller";
	case 0x01010a00:	return "IDE controller";
	case 0x01010b00:	return "IDE controller";
	case 0x01010c00:	return "IDE controller";
	case 0x01010d00:	return "IDE controller";
	case 0x01010e00:	return "IDE controller";
	case 0x01010f00:	return "IDE controller";
	case 0x01018000:	return "IDE controller";
	case 0x01018100:	return "IDE controller";
	case 0x01018200:	return "IDE controller";
	case 0x01018300:	return "IDE controller";
	case 0x01018400:	return "IDE controller";
	case 0x01018500:	return "IDE controller";
	case 0x01018600:	return "IDE controller";
	case 0x01018700:	return "IDE controller";
	case 0x01018800:	return "IDE controller";
	case 0x01018900:	return "IDE controller";
	case 0x01018a00:	return "IDE controller";
	case 0x01018b00:	return "IDE controller";
	case 0x01018c00:	return "IDE controller";
	case 0x01018d00:	return "IDE controller";
	case 0x01018e00:	return "IDE controller";
	case 0x01018f00:	return "IDE controller";
	case 0x01020000:	return "Floppy disk controller";
	case 0x01030000:	return "IPI bus controller";
	case 0x01040000:	return "RAID controller";
	case 0x01052000:	return "ATA controller with single DMA";
	case 0x01053000:	return "ATA controller with chained DMA";
	case 0x01800000:	return "Other mass storage controller";
	case 0x02000000:	return "Ethernet controller";
	case 0x02010000:	return "Token Ring controller";
	case 0x02020000:	return "FDDI controller";
	case 0x02030000:	return "ATM controller";
	case 0x02040000:	return "ISDN controller";
	case 0x02050000:	return "WorldFip controller";
	case 0x02060000:	return "PICMG 2.14 Multi Computing";
	case 0x02800000:	return "Other network controller";
	case 0x03000000:	return "VGA-compatible controller";
	case 0x03000100:	return "8514-compatible controller -- 2E8h and its aliases, 2EAh-2EFh";
	case 0x03010000:	return "XGA controller";
	case 0x03020000:	return "3D controller";
	case 0x03800000:	return "Other display controller";
	case 0x04000000:	return "Video device";
	case 0x04010000:	return "Audio device";
	case 0x04020000:	return "Computer telephony device";
	case 0x04800000:	return "Other multimedia device";
	case 0x05000000:	return "RAM";
	case 0x05010000:	return "Flash";
	case 0x05800000:	return "Other memory controller";
	case 0x06000000:	return "Host bridge";
	case 0x06010000:	return "ISA bridge";
	case 0x06020000:	return "EISA bridge";
	case 0x06030000:	return "MCA bridge";
	case 0x06040000:	return "PCI-to-PCI bridge";
	case 0x06040100:	return "Subtractive Decode PCI-to-PCI bridge";
	case 0x06050000:	return "PCMCIA bridge";
	case 0x06060000:	return "NuBus bridge";
	case 0x06070000:	return "CardBus bridge";
	case 0x06080000:	return "RACEway bridge -- Transparent mode";
	case 0x06080100:	return "RACEway bridge -- End-point mode";
	case 0x06094000:	return "Semi-transparent PCI-to-PCI bridge with the primary PCI bus side facing the system host processor";
	case 0x06098000:	return "Semi-transparent PCI-to-PCI bridge with the secondary PCI bus side facing the system host processor";
	case 0x060a0000:	return "InfiniBand-to-PCI host bridge";
	case 0x06800000:	return "Other bridge device";
	case 0x07000000:	return "Generic XT-compatible serial controller";
	case 0x07000100:	return "16450-compatible serial controller";
	case 0x07000200:	return "16550-compatible serial controller";
	case 0x07000300:	return "16650-compatible serial controller";
	case 0x07000400:	return "16750-compatible serial controller";
	case 0x07000500:	return "16850-compatible serial controller";
	case 0x07000600:	return "16950-compatible serial controller";
	case 0x07010000:	return "Parallel port";
	case 0x07010100:	return "Bi-directional parallel port";
	case 0x07010200:	return "ECP 1.X compliant parallel port";
	case 0x07010300:	return "IEEE1284 controller";
	case 0x0701fe00:	return "IEEE1284 target device (not a controller)";
	case 0x07020000:	return "Multiport serial controller";
	case 0x07030000:	return "Generic modem";
	case 0x07030100:	return "Hayes compatible modem, 16450-compatible interface";
	case 0x07030200:	return "Hayes compatible modem, 16550-compatible interface";
	case 0x07030300:	return "Hayes compatible modem, 16650-compatible interface";
	case 0x07030400:	return "Hayes compatible modem, 16750-compatible interface";
	case 0x07040000:	return "GPIB (IEEE 488.1/2) controller";
	case 0x07050000:	return "Smart Card";
	case 0x07800000:	return "Other communications device";
	case 0x08000000:	return "Generic 8259 PIC";
	case 0x08000100:	return "ISA PIC";
	case 0x08000200:	return "EISA PIC";
	case 0x08001000:	return "I/O APIC interrupt controller";
	case 0x08002000:	return "I/O(x) APIC interrupt controller";
	case 0x08010000:	return "Generic 8237 DMA controller";
	case 0x08010100:	return "ISA DMA controller";
	case 0x08010200:	return "EISA DMA controller";
	case 0x08020000:	return "Generic 8254 system timer";
	case 0x08020100:	return "ISA system timer.";
	case 0x08020200:	return "EISA system timers (two timers)";
	case 0x08030000:	return "Generic RTC controller";
	case 0x08030100:	return "ISA RTC controller";
	case 0x08040000:	return "Generic PCI Hot-Plug controller";
	case 0x08800000:	return "Other system peripheral";
	case 0x09000000:	return "Keyboard controller";
	case 0x09010000:	return "Digitizer (pen)";
	case 0x09020000:	return "Mouse controller";
	case 0x09030000:	return "Scanner controller";
	case 0x09040000:	return "Gameport controller (generic)";
	case 0x09041000:	return "Gameport controller";
	case 0x09800000:	return "Other input controller";
	case 0x0a000000:	return "Generic docking station";
	case 0x0a800000:	return "Other type of docking station";
	case 0x0b000000:	return "386";
	case 0x0b010000:	return "486";
	case 0x0b020000:	return "Pentium";
	case 0x0b100000:	return "Alpha";
	case 0x0b200000:	return "PowerPC";
	case 0x0b300000:	return "MIPS";
	case 0x0b400000:	return "Co-processor";
	case 0x0c000000:	return "IEEE 1394 (FireWire)";
	case 0x0c001000:	return "IEEE 1394 following the 1394 OpenHCI specification";
	case 0x0c010000:	return "ACCESS.bus";
	case 0x0c020000:	return "SSA";
	case 0x0c030000:	return "Universal Serial Bus (USB) following the Universal Host Controller Specification";
	case 0x0c031000:	return "Universal Serial Bus (USB) following the Open Host Controller Specification";
	case 0x0c032000:	return "USB2 host controller following the Intel Enhanced Host Controller Interface";
	case 0x0c038000:	return "Universal Serial Bus with no specific programming interface";
	case 0x0c03fe00:	return "USB device (not host controller)";
	case 0x0c040000:	return "Fibre Channel";
	case 0x0c050000:	return "SMBus (System Management Bus)";
	case 0x0c060000:	return "InfiniBand";
	case 0x0c070000:	return "IPMI SMIC Interface";
	case 0x0c070100:	return "IPMI Kybd Controller Style Interface";
	case 0x0c070200:	return "IPMI Block Transfer Interface";
	case 0x0c080000:	return "SERCOS Interface Standard (IEC 61491)";
	case 0x0c090000:	return "CANbus";
	case 0x0d000000:	return "iRDA compatible controller";
	case 0x0d010000:	return "Consumer IR controller";
	case 0x0d100000:	return "RF controller";
	case 0x0d110000:	return "Bluetooth";
	case 0x0d120000:	return "Broadband";
	case 0x0d800000:	return "Other type of wireless controller";
	case 0x0e000000:	return "Intelligent I/O (I2O)";
	case 0x0f010000:	return "TV";
	case 0x0f020000:	return "Audio";
	case 0x0f030000:	return "Voice";
	case 0x0f040000:	return "Data";
	case 0x10000000:	return "Network and computing en/decryption";
	case 0x10100000:	return "Entertainment en/decryption";
	case 0x10800000:	return "Other en/decryption";
	case 0x11000000:	return "DPIO modules";
	case 0x11010000:	return "Performance counters";
	case 0x11100000:	return "Communications synchronization plus time and frequency test/measurement";
	case 0x11200000:	return "Management card";
	case 0x11800000:	return "Other data acquisition/signal processing controllers";
	case 0xff000000:	return "Device does not fit in any defined classes";
	}
	sprintf(szBuf, "%08lX", Reg);
	return szBuf;
}

//-------------------------------------------------------------------
// Main routine : Enum PCI bus devices
//-------------------------------------------------------------------
#define MAX_BUS				256
#define	PCI_MAX_DEVICES		32
#define	PCI_MAX_FUNCTION	8

void main(int argc, char *argv[])
{
	unsigned int	BusNumber;
	unsigned char	DevNumber;
	unsigned char	FunNumber;
	SLOT_NUMBER		SlotNumber;
	unsigned long	RegData[2];
	unsigned long	RegList[30];
	int				i;
	//-----------------------
	// Parameter check
	//-----------------------
	if (argc >= 2 && strcmp(argv[1], "/?") == 0) {
		printf("CONTEC PCI Information Utility\n");
		printf("This command show configuration register information.\n");
		return;
	}
	//-----------------------
	// Search for Bus number
	//-----------------------
	printf("=================================================\n");
	printf("GETCONF : CONTEC PCI Information Utility\n");
	printf("=================================================\n");
	for(BusNumber = 0; BusNumber < MAX_BUS; BusNumber++) {
		//--------------------------
		// Search for Device number
		//--------------------------
		for(DevNumber = 0; DevNumber < PCI_MAX_DEVICES; DevNumber++) {
			SlotNumber.num.bits.DeviceNumber = DevNumber;
			//----------------------------
			// Search for Function number
			//----------------------------
			SlotNumber.num.bits.FunctionNumber = 0;
			for (FunNumber = 0; FunNumber < PCI_MAX_FUNCTION; FunNumber++) {
				SlotNumber.num.bits.FunctionNumber = FunNumber;
				RegData[0] = ReadConfigReg((unsigned char)BusNumber, SlotNumber.num.AsChar, 0);
				//----------------------------
				// Search on Function number
				//----------------------------
				if (RegData[0] != 0 && RegData[0] != 0xffff) {
					for (i=0; i<16; i++) {
						RegData[0] = ReadConfigReg((unsigned char)BusNumber, SlotNumber.num.AsChar, i*4);
						RegData[1] = ReadConfigReg((unsigned char)BusNumber, SlotNumber.num.AsChar, i*4+2);
						RegList[i] = RegData[0] | (RegData[1] << 16);
					}
					printf("------------------------------\n");
					printf("BusNumber = %X, SlotNumber = %X\nClass = %s\n", BusNumber, (int)SlotNumber.num.AsChar, GetPciClassName(RegList[2]));
					for (i=0; i<16; i++) {
						printf("%08X : %08lX\n", i*4, RegList[i]);
					}
				}
				//---------------------------------
				// If (Register[0x0e] & 0x80) then
				// search for next Function number
				//---------------------------------
				if (FunNumber == 0 && ((RegList[3] >> 16) & 0x80) == 0x00) {
					break;
				}
			}
		}
	}
}
