//------------------------------------------------------------------
// CONTEC PCI RESOURCE RECOGNIZING TOOL FOR MS-DOS
// GETPCI.C
//												CONTEC Co.,Ltd
//												2002.04.18
//------------------------------------------------------------------
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include "getpci.h"
//------------------------------------------------------------------
// Constants of PCI bus information 
//------------------------------------------------------------------
#define		PciBiosInt_						0x1a
#define		PciFunctionId_					0xb1
#define		GetPciBiosPresentStatus_		0x01
#define		FindPciDevice_					0x02
#define		ReadPciConfigurationWord_		0x09
#define		ConfigRegDeviceIdOffset_		0x02
#define		ConfigRegRevisionOffset_		0x08
#define		ConfigRegBaseAddressOffset_		0x10
#define		ConfigRegInterruptLineOffset_	0x3c

//-------------------------------------------------------------------
// Sub routine : Read configuration register word.
// unsigned int ReadConfigReg(unsigned char PciBusNumber, unsigned char PciSlotNumber, int Offset)
// Parameters:
//			PciBusNumber	:	Bus number of read device
//			PciSlotNumber	:	Set of device number and function number
//			Offset			:	Offset of read register
// return:
//			register data, 0 if error.
//-------------------------------------------------------------------
unsigned int ReadConfigReg(unsigned char PciBusNumber, unsigned char PciSlotNumber, int Offset)
{
	union	REGS	IRegs;
	union	REGS	ORegs;
	struct	SREGS	SRegs;
	static 	unsigned char MaxBusNumber = 0;
	//--------------------------------------
	// get PCI BIOS present status
	//--------------------------------------
	if (MaxBusNumber != 0 && PciBusNumber > MaxBusNumber) {
		return 0;
	}
	IRegs.h.ah = PciFunctionId_;
	IRegs.h.al = GetPciBiosPresentStatus_;
	int86x( PciBiosInt_, &IRegs, &ORegs, &SRegs );
	if (MaxBusNumber == 0) {
		MaxBusNumber = ORegs.h.cl;
	}
	if ( !ORegs.h.ah && ( ORegs.x.dx == 0x4350 ) ) {
		//--------------------------------------
		// Read Configuration Register
		//--------------------------------------
		IRegs.h.ah = PciFunctionId_;
		IRegs.h.al = ReadPciConfigurationWord_;
		IRegs.h.bh = (unsigned char)PciBusNumber;
		IRegs.h.bl = (unsigned char)PciSlotNumber;
		IRegs.x.di = Offset;
		int86x( PciBiosInt_, &IRegs, &ORegs, &SRegs );
		if (ORegs.h.ah == 0) {
			return ORegs.x.cx;
		} else {
			return 0;
		}
	} else {
		return 0;
	}
}

//-------------------------------------------------------------------
// Sub routine : Get resource of one PCI bus board.
// int GetPciResource(	unsigned short VenderID, unsigned short DeviceID, unsigned short BoardID,
//					unsigned short *IoAddr, unsigned short *IoNum,
//					unsigned short *Irq,
//					unsigned long *MemAddr, unsigned short *MemNum)
// Parameters:
//			VenderID		:	VenderID
//			DeviceID		:	DeviceID
//			BoardID			:	BoardID(RevisionID)
//			IoAddr			:	Address of IoAddr[6]
//			IoNum			:	Address of I/O address number
//			Irq				:	Address of IRQ(if none, return 0xff);
//			MemAddr			:	Address of MemAddr[6]
//			MemNum			:	Address of Memory address number
//			
// return:
//			0				:	Correct
//			-1				:	Not present
//			-2				:	Board ID conflict
//-------------------------------------------------------------------
#define	BOARD_NUNBER_MAX		16
int GetPciResource(	unsigned short VenderID, unsigned short DeviceID, unsigned short BoardID,
					unsigned short *IoAddr, unsigned short *IoNum,
					unsigned short *Irq,
					unsigned long *MemAddr, unsigned short *MemNum)
{
	union	REGS	IRegs;
	union	REGS	ORegs;
	struct	SREGS	SRegs;
	int		i;
	int		NumberOfFound;
	char	BusNumber[BOARD_NUNBER_MAX];
	char	SlotNumber[BOARD_NUNBER_MAX];
	unsigned long	TmpRead0, TmpRead1;
	unsigned long	TmpIoMem[6];
	unsigned short	TmpIrq;
	unsigned short	TmpBoardID;
	short			BoardIDFound;
	short			Index;
	//--------------------------------------
	// get PCI BIOS present status
	//--------------------------------------
	IRegs.h.ah = PciFunctionId_;
	IRegs.h.al = GetPciBiosPresentStatus_;
	int86x( PciBiosInt_, &IRegs, &ORegs, &SRegs );
	if ( ORegs.h.ah || ( ORegs.x.dx != 0x4350 )) {
		return -1;
	}
	//--------------------------------------
	// Search PCI device
	//--------------------------------------
	NumberOfFound = 0;
	for (i=0; i<BOARD_NUNBER_MAX; i++ ) {
		IRegs.h.ah = PciFunctionId_;
		IRegs.h.al = FindPciDevice_;
		IRegs.x.cx = DeviceID;
		IRegs.x.dx = VenderID;
		IRegs.x.si = i;
		int86x( PciBiosInt_, &IRegs, &ORegs, &SRegs );
		if ( ORegs.h.ah ) {
			break;
		}
		BusNumber[NumberOfFound] = ORegs.h.bh;
		SlotNumber[NumberOfFound] = ORegs.h.bl;
		NumberOfFound++;
	}
	//--------------------------------------
	// Get Revision ID(=BoardID)
	//--------------------------------------
	BoardIDFound = 0;
	for (i=0; i<NumberOfFound; i++) {
		TmpBoardID = ReadConfigReg(BusNumber[i], SlotNumber[i], ConfigRegRevisionOffset_);
		if (TmpBoardID == BoardID) {
			Index = i;
			BoardIDFound++;
		}
	}
	if (BoardIDFound == 0) {
		return -1;
	}
	if (BoardIDFound != 1) {
		return -2;
	}
	//--------------------------------------
	// Get Resource
	//--------------------------------------
	// address
	for (i=0; i<6; i++) {
		TmpRead0 = ReadConfigReg(BusNumber[Index], SlotNumber[Index], ConfigRegBaseAddressOffset_ + i*4);
		TmpRead1 = ReadConfigReg(BusNumber[Index], SlotNumber[Index], ConfigRegBaseAddressOffset_ + i*4+2);
		TmpIoMem[i] = TmpRead0 | (TmpRead1 << 16);
	}
	// IRQ
	TmpIrq = ReadConfigReg(BusNumber[Index], SlotNumber[Index], ConfigRegInterruptLineOffset_);
	if (TmpIrq & 0xff00) {
		TmpIrq = TmpIrq & 0xff;
	} else {
		TmpIrq = 0xff;
	}
	//--------------------------------------
	// Retutrn Resource
	//--------------------------------------
	if (IoAddr != NULL && IoNum != NULL) {
		*IoNum = 0;
		for (i=0; i<6; i++) {
			if (TmpIoMem[i] & 0x01) {
				IoAddr[*IoNum] = (unsigned short)(TmpIoMem[i] & 0xfffe);
				*IoNum = *IoNum + 1;
			}
		}
	}
	if (Irq != NULL) {
		*Irq = (unsigned short)TmpIrq;
	}
	if (MemAddr != NULL && MemNum != NULL) {
		*MemNum = 0;
		for (i=0; i<6; i++) {
			if (!(TmpIoMem[i] &0x01) && TmpIoMem[i] != 0) {
				MemAddr[*MemNum] = TmpIoMem[i];
				*MemNum = *MemNum + 1;
			}
		}
	}
	return 0;
}
