#include <stdio.h>
#include <stdlib.h>
#include "../../inc/cgpib.h"					// GPIB header file

#define	MAXLEN	1024

short	yradr;								// Other Address
long	Ret;								// Return value
short	DevId;								// DevID
short	EqpId;								// EqpId
short	Delim;								// Delimiter
long	TimeOut;							// Time Out Count
long	srlen;								// SendData and RecData Number
char	srdata[MAXLEN + 2];					// SendData and RecData Buffer

//================================================================
//	Data Send Routine
//================================================================
void Talk(void)
{
	printf("*** Input Send Data *** \n");

	//-------------------------------------
	//	Number setup of send data
	//-------------------------------------
	do{
		printf("SEND DATA = ");
		srlen = 0;
		while((srdata[srlen] = (char)getchar()) != 0xa){
			srlen++;
		}
		srdata[srlen] = 0;
		if(srlen > MAXLEN){
			printf("Data is long !  Please input data again. \n");
		}
		if(srlen == 0){
			printf("There is no data.  Please input data again. \n");
		}
	}while(srlen > MAXLEN || srlen == 0);

	printf("\n");

	//-------------------------------------
	//	Data Send
	//-------------------------------------
	Ret = GpibSendData(EqpId, &srlen, srdata);
	if(Ret != 0){
		printf("%s = %ld\n", "GpibSendData", Ret);
		return;
	}

	//-------------------------------------
	//	Output
	//-------------------------------------
	printf("length = %ld\n", srlen);
}

//================================================================
//	Data  Receive  Routine
//================================================================
void Listen(void)
{
	char	srdatabuf[MAXLEN + 2];					// RecData Buffer

	printf("*** Receive Data ***\n");
	srlen = MAXLEN;									// Rec Data Length
	printf("Now Data Receiving ...\n");

	//-------------------------------------
	//	Receive Data
	//-------------------------------------
	Ret = GpibRecData(EqpId, &srlen, srdatabuf);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibRecData", Ret);
		return;
	}
	strncpy(srdata, srdatabuf, srlen);
	
	//-------------------------------------
	//	Output
	//-------------------------------------
	printf("data\n");
	printf("%s\n", srdata);
	printf("length = %ld\n", srlen);
}

//================================================================
//	Serial  Polling  Routine
//================================================================
void Polling(void)
{
	short	Stb;							// Stb
	short	SrqLine;						// Srq Line

	//-------------------------------------
	//	Serial Polling
	//-------------------------------------
	printf("*** Serial polling ***\n");
	
	Ret = GpibSPoll(EqpId, &Stb, &SrqLine);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibSPoll", Ret);
		return;
	}

	printf("Status byte = %x : SRQ Line = %d\n", Stb, SrqLine);
}
//================================================================
//	Group Execute Trigger  Routine
//================================================================
void Trigger(void)
{

	//-------------------------------------
	//	Send Trigger
	//-------------------------------------
	printf("*** Group Execute Trigger ***\n");
	
	Ret = GpibSendTrigger(EqpId);
	if (Ret != 0){
		printf( "%s = %ld\n", "GpibSendTrigger", Ret);
		return;
	}
}

//================================================================
//	Input Set Value by Data File 'master.dat'
//================================================================
int menu(void)
{
	FILE	*fp;
	char	cBuf[256];

	if ((fp = fopen("master.dat", "r")) == NULL) {
		printf("File[master.dat] open error !! \n");
		return -1;
	} else {
		printf("Other system address : ");
		fscanf(fp, "%s", cBuf);
		yradr = atoi(cBuf);
		printf("%d\n", yradr);
		
		printf("Delimiter code (0:null / 1:CR+LF / 2:CR / 3:LF) : ");
		fscanf(fp, "%s", cBuf);
		Delim = atoi(cBuf);
		printf("%d\n", Delim);

		printf("Time out (0:no use / 1-65535:[*10msec]) : ");
		fscanf(fp, "%s", cBuf);
		TimeOut = atoi(cBuf);
		printf("%ld\n", TimeOut);

		printf("\n");

		fclose(fp);
	}

	return 0;
}

//================================================================
//	Main Routine
//================================================================
int main(void)
{	
	char	DeviceName[255];					// Device Name

	//-------------------------------------
	//	Check Device Name
	//-------------------------------------
	printf("\nPlease input a device name: ");
	scanf("%s", DeviceName);

	//-------------------------------------
	//	GPIB Initial
	//-------------------------------------
	Ret = GpibInit(DeviceName, &DevId);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibInit", Ret);
		return -1;
	}
	if (menu()) {
		goto MAIN_END;
	}

	//-------------------------------------
	//	Set Equipment ID
	//-------------------------------------
	Ret = GpibSetEquipment(DevId, &EqpId, yradr, 0, Delim);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibSetEquipment", Ret);
		if (Ret == 20495) {
			printf( "There is no connection apparatus or the power supply is not on.\n");
		}
		goto MAIN_END;
	}

	//-------------------------------------
	//	Set Timeout
	//-------------------------------------
	Ret = GpibSetTimeOut(DevId, TimeOut);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibSetTimeOut", Ret);
		goto MAIN_END;
	}

	//-------------------------------------
	//	Waiting... Input Key Routine
	//-------------------------------------
	while (1) {
		printf("*** Send='S', Receive='R', Serial poll='P', Group Execute Trigger='T', End='Q' *** \n");
		switch (getchar()) {
		case 'S':
		case 's':
			Talk(); break;
		case 'R':
		case 'r':
			Listen(); break;
		case 'P':
		case 'p':
			Polling(); break;
		case 'T':
		case 't':
			Trigger(); break;
		case 'Q':
		case 'q':
			goto MAIN_END; break;
		}
	}
	
MAIN_END:
	//-------------------------------------
	//	Gpib Exit
	//-------------------------------------
	Ret = GpibExit(DevId);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibExit", Ret);
		return -1;
	}
	printf("*** E N D ***\n");
	return 0;
}

