#include <stdio.h>
#include <stdlib.h>
#include "../../inc/cgpib.h"					// GPIB header file
#include "../../inc/kbhit.h"					// kbhit()

#define MAXLEN 255

short	yradr = 2;								// Other Address
short	Delim = 1;								// Delimiter
char	Sendbuf[] = "F1R0M1IT1DL0S0RE7BZ1";		// Listenner Format
long	TimeOut = 1000;							// Time Out Count(1000*10[msec])

long	Ret;									// Return value
short	DevId;									// DevID
short	EqpId;									// EqpID
long	srlen;									// SendData and RecData Number
char	srdata[MAXLEN + 2];						// SendData and RecData Buffer

//================================================================
//	Data Send Routine
//================================================================
void Talk(void)
{
	srlen = strlen(Sendbuf);					// Send Data Length
	//-------------------------------------
	//	Data Send
	//-------------------------------------
	Ret = GpibSendData(EqpId, &srlen, Sendbuf);
	if (Ret != 0){
		printf("GpibSendData = %ld\n", Ret);
		return;
	}
	//-------------------------------------
	//	Output
	//-------------------------------------
	printf("Send Data = %ld\n", srlen);
	return;
}

//================================================================
//	Data Receive Routine
//================================================================
void Listen(void)
{
	char	srdatabuf[MAXLEN + 2];					// RecData Buffer

	//-------------------------------------------
	//	Receive Data 
	//-------------------------------------------
	srlen = MAXLEN;									// Rec Data Length
	Ret = GpibRecData(EqpId, &srlen, srdatabuf);
	if (Ret != 0){
		printf("GpibRecData = %ld\n", Ret);
		return;
	}
	strncpy(srdata, srdatabuf, srlen);

	//-------------------------------------
	//	output
	//-------------------------------------
	printf(" Receive data = %s\n", srdata);
	return;
}

//================================================================
//	Main Routine
//================================================================
int main(void)
{
	char	DeviceName[255];					// Device Name
	long	SRQflag;							// SRQ Flag
	char	key[256];							// Keyboard input
	short	Stb;								// Stb
	short	SrqLine;							// Srq Line

	key[0] = 0;

	printf("\n*** Advantest Digital Multimeter R6871E [Stop: Enter Key] ***\n");

	//-------------------------------------
	//	Check Device Name
	//-------------------------------------
	printf("\nPlease input a device name: ");
	scanf("%s", DeviceName);

	//-------------------------------------
	//	GPIB Initial
	//-------------------------------------
	Ret = GpibInit(DeviceName, &DevId);
	if (Ret != 0){
		printf( "%s = %ld\n", "GpibInit", Ret);
		return -1;
	}

	//-------------------------------------
	//	Set Equipment ID
	//-------------------------------------
	Ret = GpibSetEquipment(DevId, &EqpId, yradr, 0, Delim);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibSetEquipment", Ret);
		goto MAIN_END;
	}

	//----------------------------------------
	//	Set Timeout
	//----------------------------------------
	Ret = GpibSetTimeOut(DevId, TimeOut);
	if (Ret != 0){
		printf( "%s = %ld\n", "GpibSetTimeOut", Ret);
		goto MAIN_END;
	}

	Talk();										//Send Data

	while(1){
		//-------------------------------------------
		//	Group Execute Trigger
		//-------------------------------------------
		Ret = GpibSendTrigger(EqpId);
		if (Ret != 0){
			printf( "%s = %ld\n", "GpibSendTrigger", Ret);
			goto MAIN_END;
		}

		//-------------------------------------------
		//	Srq Check 
		//-------------------------------------------
		do{
			Ret = GpibGetStatus(DevId, 0x12, &SRQflag);		//SRQ Check
			if (Ret!= 0){
			printf( "%s = %ld\n", "GpibGetStatus", Ret);
				goto MAIN_END;
			}
		}while(SRQflag != 1);				// SRQ On ?

		//-------------------------------------------
		//	Serial Poll
		//-------------------------------------------
		Ret = GpibSPoll(EqpId, &Stb, &SrqLine);
		if(Ret != 0){
			printf( "%s = %ld\n", "GpibSPoll", Ret);
			goto MAIN_END;
		}
		printf("Status byte = %x : SRQ Line = %d\n", Stb, SrqLine);

		Listen();											// Receive Data

		if(kbhit() != 0) {
			printf("End?  ('Y'/'N')\n");
			scanf("%s", key);
			if ((key[0] == 'Y') || (key[0] == 'y')) goto MAIN_END;
		}
	}
	
MAIN_END:
	//-------------------------------------
	//	GPIB Exit
	//-------------------------------------
	Ret = GpibExit(DevId);
	if(Ret != 0){
		printf("%s = %ld\n", "GpibExit", Ret);
		return -1;
	}
	printf("*** E N D ***\n");
	return 0;
}

