#include <stdio.h>
#include <stdlib.h>
#include "../../inc/cgpib.h"						// GPIB header file

#define	MAXLEN						1024		// Data Max Length

long	Ret;									// Return value
short	DevId;									// DevID
short	Delim;									// Delimiter
long	TimeOut;								// Time Out Count
long	srlen;									// SendData and RecData Number
char	srdata[MAXLEN + 2];						// Data Buffer

//================================================================
//	Data Send Routine
//================================================================
void Talk(void)
{
	long	TAflag;								// TA Flag

	printf("*** Input Send Data *** \n");

	//-------------------------------------
	//	Number setup of send data
	//-------------------------------------
	do{
		printf("SEND DATA = ");
		srlen = 0;
		while((srdata[srlen] = (char)getchar()) != 0xa){	// LF
			srlen++;
		}
		srdata[srlen] = 0;
		if(srlen > MAXLEN){
			printf("Data is long !  Please input data again. \n");
		}
		if(srlen == 0){
			printf("There is no data.  Please input data again. \n");
		}
	}while(srlen > MAXLEN || srlen == 0);

	printf("\n");

	//-------------------------------------
	//	Wait TA
	//-------------------------------------
	TAflag = 0;
	do{
		Ret = GpibGetStatus(DevId, 0x13, &TAflag);		// TA Check
		if(Ret != 0){
			printf( "%s = %ld\n", "GpibGetStatus", Ret);
			return;
		}
	}while(TAflag != 1);

	//-------------------------------------
	//	Data Send
	//-------------------------------------
	Ret = GpibSendData(DevId, &srlen, &srdata[0]);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibSendData", Ret);
		return;
	}

	//-------------------------------------
	//	Output
	//-------------------------------------
	printf("length = %ld\n", srlen);
}

//================================================================
//	Data Receive Routine
//================================================================
void Listen(void)
{
	long	LAflag;									// LA Flag
	char	srdatabuf[MAXLEN + 2];					// RecData Buffer

	printf("*** Receive Data ***\n");
	srlen = MAXLEN;								// Rec Data Length
	printf("Now Data Receiving ...\n");
	
	//-------------------------------------
	//	Wait LA
	//-------------------------------------
	LAflag = 0;
	do{
		Ret = GpibGetStatus(DevId, 0x14, &LAflag);		// LA Check
		if(Ret != 0){
			printf( "%s = %ld\n", "GpibGetStatus", Ret);
			return;
		}
	}while(LAflag != 1);

	//-------------------------------------
	//Receive Data
	//-------------------------------------
	Ret = GpibRecData(DevId, &srlen, srdatabuf);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibRecData", Ret);
		return;
	}
	strncpy(srdata, srdatabuf, srlen);
	
	//-------------------------------------
	//	output
	//-------------------------------------	
	printf("data\n");
	printf("%s\n", srdata);
	printf("length = %ld\n", srlen);
}

//================================================================
//	Service  Request  Routine
//================================================================
void Service(void)
{
	short SrqSend;								// SRQ Send
	short Stb;									// Status Bit
	short SPoll;								// Serial poll
	int	intdata;

	//-------------------------------------
	//	Set Status Byte
	//-------------------------------------
	printf("*** Input status byte (SRQ send) ***\n");
	printf("Status Byte = ");
	
	do{
		scanf("%x", &intdata);
		Stb = (short)intdata;
		getchar();	// delete LF
		
		//flag = 0;
		if((Stb < 64) || (Stb > 255)){
			printf("Error!!  40H <= Stb <= ffH\nStatus Byte = ");
		}
	}while((Stb < 64) || (Stb > 255));
	
	//-------------------------------------
	//	Send SRQ
	//-------------------------------------
	SrqSend = 1;								// flag of send SRQ
	Ret = GpibSendSRQ(DevId, SrqSend, Stb);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibSendSRQ", Ret);
		return;
	}

	//-------------------------------------
	//	Check Status Byte
	//-------------------------------------
	do{
		Ret = GpibCheckSPoll(DevId, &SPoll, &Stb);
		if(Ret != 0){
			printf( "%s = %ld\n", "GpibCheckSPoll", Ret);
			return;
		}
		printf(" ---------- loop1 --------- %x ----\n", Stb);
	}while(SPoll == 0);

	printf("Serial Poll was carried out. \n");

}

//================================================================
//	TA LA Check
//================================================================
void CheckTALA(void)
{
	long	TAflag;								// TA Flag
	long	LAflag;								// LA Flag

	//-------------------------------------
	//	TA Check
	//-------------------------------------
	Ret = GpibGetStatus(DevId, 0x13, &TAflag);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibGetStatus", Ret);
		return;
	}
	if(TAflag == 1) {
		printf("You are talker. \n");
		return;
	}
	//-------------------------------------
	//	LA Check
	//-------------------------------------
	Ret = GpibGetStatus(DevId, 0x14, &LAflag);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibGetStatus", Ret);
		return;
	}
	if(LAflag == 1) {
		printf("You are listenner. \n");
		return;
	}
	printf("You are not talker and listenner. \n");
}


//================================================================
//	Input Set Value by Data File 'slave.dat'
//================================================================
int menu(void)
{
	FILE	*fp;
	char	cBuf[256];

	if ((fp = fopen("slave.dat", "r")) == NULL) {
		printf("File[slave.dat] open error !! \n");
		return	-1;
	} else {
		printf("Delimiter code (0:null / 1:CR+LF / 2:CR / 3:LF) : ");
		fscanf(fp, "%s", cBuf);
		Delim = atoi(cBuf);
		printf("%d\n", Delim);

		printf("Time out (0:no use / 1-65535:[*10msec]) : ");
		fscanf(fp, "%s", cBuf);
		TimeOut = atoi(cBuf);
		printf("%ld\n", TimeOut);

		printf("\n");

		fclose(fp);
	}

	return	0;
}

//================================================================
// Main Routine
//================================================================
int main()
{
	char	DeviceName[255];					// Device Name

	//-------------------------------------
	//	Check Device Name
	//-------------------------------------
	printf("\nPlease input a device name: ");
	fgets(DeviceName, sizeof(DeviceName), stdin);		// get the input data
	DeviceName [strlen(DeviceName)-1] = '\0';
		
	//-------------------------------------
	//	GPIB Initial
	//-------------------------------------
	Ret = GpibInit(DeviceName, &DevId);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibInit", Ret);
		return -1;
	}

	// Read 'slave.dat' 
	if (menu()) {
		goto MAIN_END;
	}

	//-------------------------------------
	//	Set Delimiter
	//-------------------------------------	
	Ret = GpibSetDelim(DevId, Delim, 1, 0);	
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibSetDelim", Ret);
		goto MAIN_END;
	}

	//-------------------------------------
	//	Set Timeout
	//-------------------------------------
	Ret = GpibSetTimeOut(DevId, TimeOut);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibSetTimeOut", Ret);
		goto MAIN_END;
	}

	//-------------------------------------
	//	Waiting... Input Key Routine
	//-------------------------------------
	while(1) {
		printf("*** Send='S', Receive='R', Service Request='P', TA LA Check='C', End='Q' *** \n");
		switch (getchar()) {
		// Send data
		case 'S':
		case 's':
			Talk();
			break;
		// Receive data
		case 'R':
		case 'r':
			Listen();
			break;
		// Service request
		case 'P':
		case 'p':
			Service();
			break;
		// TA LA Check
		case 'C':
		case 'c':
			CheckTALA();
			break;
		// Exit
		case 'Q':
		case 'q':
			goto MAIN_END;
			break;
		}
		
	}
	
MAIN_END:
	//-------------------------------------
	//	Gpib Exit
	//-------------------------------------
	Ret = GpibExit(DevId);
	if(Ret != 0){
		printf( "%s = %ld\n", "GpibExit", Ret);
		return -1;
	}
		printf("*** E N D ***\n");
	return 0;
}
